var util = require('util')
  , EOL = require('os').EOL
  , NIL = '(nil)'
  , EMPTY = '(empty list or set)'
  , assert = require('assert');

/**
 *  Repeat a string by amount.
 */
function repeat(str, amount) {
  var s = ''
    , i;
  for(i = 0;i < amount;i++) {
    s += str;
  }
  return s;
}

/**
 *  Format an error reply pair as plain text.
 *
 *  Suitable for a client command line interface.
 */
function Format() {}

function convert(err, reply, str, indent) {
  var i
    , item
    , err
    , len
    , prefix
    , space = ' '
    , padding;

  indent = indent || 0;
  str = str || '';
  if(err) {
    str = util.format('(error) %s', err.message);
  }else{
    if(reply === null) {
      str = NIL;
    }else if((reply instanceof String)) {
      str = reply.toString();
    }else if(typeof reply === 'string') {
      str = util.format('"%s"', reply);
    }else if(typeof reply === 'number') {
      str = util.format('(integer) %s', reply);
    }else if(Array.isArray(reply)) {
      if(!reply.length) {
        str = EMPTY;
      }else{
        // maximum padding length: +2 accounts for ) and single space
        len = ('' + reply.length).length + 2;
        for(i = 0;i < reply.length;i++) {
          item = reply[i];
          prefix = (i + 1) + ')';
          padding = repeat(space, len - prefix.length);
          err = (item instanceof Error);

          // indent for nested arrays
          if(i > 0 && indent) {
            str += repeat(space, indent);
          }

          str += prefix
            + padding
            + convert(err ? item : null, !err ? item : undefined,
              null,
              !Array.isArray(item) ? 0 : indent + len);
        }
        // prevent additional trailing newline
        return str;
      }
    }else{
      assert(false, 'bad format reply type: ' + typeof reply);
    }
  }
  if(!/\n/.test(str)) str += EOL;
  return str;
}

Format.prototype.convert = convert;

module.exports = Format;
